part of datagrid;

/// A cell renderer which displays the [DateTime] value in the cell.
///
/// This renderer is typically used for [GridDateTimeColumn].
class GridCellDateTimeRenderer<T1, T2>
    extends GridVirtualizingCellRendererBase<Text, Widget> {
  /// Creates the [GridCellDateTimeRenderer] for [SfDataGrid] widget.
  GridCellDateTimeRenderer(_DataGridStateDetails dataGridStateDetails) {
    this._dataGridStateDetails = dataGridStateDetails;
  }

  @override
  void setCellStyle(DataCellBase dataCell) {
    if (dataCell != null) {
      final _DataGridSettings dataGridSettings = this._dataGridStateDetails();
      var currentRowIndex = _GridIndexResolver.resolveToRecordIndex(
          dataGridSettings, dataCell.rowIndex);
      var cellValue = dataGridSettings.source
          .getCellValue(currentRowIndex, dataCell.gridColumn.mappingName);
      var dateTimeValue = _getDateTimeValue(cellValue);
      dataCell.cellValue = cellValue;
      dataCell._displayText = dateTimeValue != null
          ? dataCell.gridColumn.getFormattedValue(dateTimeValue)
          : null;
      super.setCellStyle(dataCell);
    }
  }

  DateTime _getDateTimeValue(dynamic cellValue) {
    if (cellValue == null) {
      return null;
    }
    return DateTime.tryParse(cellValue.toString());
  }
}
