part of datagrid;

/// A cell renderer which displays the numeric value in the cell.
///
/// This renderer is typically used for [GridNumericColumn].
class GridCellNumericTextFieldRenderer<T1, T2>
    extends GridVirtualizingCellRendererBase<Text, TextField> {
  /// Creates the [GridCellNumericTextFieldRenderer] for [SfDataGrid] widget.
  GridCellNumericTextFieldRenderer(_DataGridStateDetails dataGridStateDetails) {
    this._dataGridStateDetails = dataGridStateDetails;
  }

  @override
  void setCellStyle(DataCellBase dataCell) {
    if (dataCell != null) {
      final _DataGridSettings dataGridSettings = this._dataGridStateDetails();
      var currentRowIndex = _GridIndexResolver.resolveToRecordIndex(
          dataGridSettings, dataCell.rowIndex);
      var cellValue = dataGridSettings.source
          .getCellValue(currentRowIndex, dataCell.gridColumn.mappingName);
      double numericValue = _getNumericValue(cellValue);
      dataCell.cellValue = cellValue;
      dataCell._displayText = numericValue != null
          ? dataCell.gridColumn.getFormattedValue(cellValue)
          : null;
      super.setCellStyle(dataCell);
    }
  }

  double _getNumericValue(dynamic cellValue) {
    if (cellValue == null) {
      return null;
    }
    return double.tryParse(cellValue.toString());
  }
}
