part of datagrid;

/// A cell renderer which displays the header text in the columns.
class GridHeaderCellRenderer
    extends GridVirtualizingCellRendererBase<Container, GridCell> {
  /// Creates the [GridHeaderCellRenderer] for [SfDataGrid] widget.
  GridHeaderCellRenderer(_DataGridStateDetails dataGridStateDetails) {
    this._dataGridStateDetails = dataGridStateDetails;
  }

  @override
  void onInitializeDisplayWidget(DataCellBase dataCell, Container view) {
    if (dataCell != null) {
      var label = Text(
        dataCell.cellValue,
        key: dataCell._key,
        softWrap: false,
        overflow: TextOverflow.ellipsis,
        textScaleFactor: 1.0,
        style: dataCell._cellStyle?.textStyle ??
            this
                ._dataGridStateDetails()
                .dataGridThemeData
                .headerStyle
                .textStyle,
      );

      dataCell._columnElement = GridHeaderCell(
        key: dataCell._key,
        dataCell: dataCell,
        child: ExcludeSemantics(child: label, excluding: true),
        alignment: dataCell.gridColumn.headerTextAlignment,
        padding: dataCell.gridColumn.padding,
        backgroundColor: dataCell._cellStyle?.backgroundColor ??
            this
                ._dataGridStateDetails()
                .dataGridThemeData
                .headerStyle
                .backgroundColor,
        isDirty: this._dataGridStateDetails().container._isDirty ||
            dataCell._isDirty,
      );

      label = null;
    }
  }

  @override
  void setCellStyle(DataCellBase dataCell) {
    if (dataCell != null) {
      dataCell.cellValue =
          dataCell.gridColumn.headerText ?? dataCell.gridColumn.mappingName;
      dataCell._cellStyle = DataGridHeaderCellStyle(
          backgroundColor: dataCell.gridColumn.headerStyle?.backgroundColor ??
              Colors.transparent,
          // Uncomment the below code if the mentioned report has resolved from framework side
          // https://github.com/flutter/flutter/issues/29702
          //this._dataGridStateDetails().dataGridThemeData?.headerStyle?.backgroundColor,
          textStyle: dataCell.gridColumn.headerStyle?.textStyle ??
              this
                  ._dataGridStateDetails()
                  .dataGridThemeData
                  ?.headerStyle
                  ?.textStyle);
    }
  }
}
