part of datagrid;

/// Holds the arguments for the [SfDataGrid.onQueryRowStyle] callback.
class QueryRowStyleArgs {
  /// Creates the [QueryRowStyleArgs] with the specified [rowIndex].
  QueryRowStyleArgs({@required int rowIndex}) : this._rowIndex = rowIndex;

  /// An index of a row in which the style is applied.
  int get rowIndex => this._rowIndex;
  int _rowIndex;

  /// Decides how the style is applied when selection and row style are applied.
  StylePreference get stylePreference => this._stylePreference;
  StylePreference _stylePreference = StylePreference.selection;

  /// Decides how the style is applied when selection and row style are applied.
  set stylePreference(StylePreference stylePreference) {
    this._stylePreference = stylePreference;
  }
}

/// Holds the arguments for the [SfDataGrid.onQueryCellStyle] callback.
class QueryCellStyleArgs {
  /// Creates the [QueryRowStyleArgs] with the specified [gridColumn],
  /// [rowIndex], [columnIndex], [cellValue] and [displayText].
  QueryCellStyleArgs(
      {@required GridColumn gridColumn,
      @required int rowIndex,
      @required int columnIndex,
      @required Object cellValue,
      @required String displayText})
      : this._column = gridColumn,
        this._rowIndex = rowIndex,
        this._columnIndex = columnIndex,
        this._cellValue = cellValue,
        this._displayText = displayText;

  /// A [GridColumn] in which the style is applied.
  GridColumn get column => this._column;
  GridColumn _column;

  /// A row index of a cell.
  int get rowIndex => this._rowIndex;
  int _rowIndex;

  /// A column index of a cell.
  int get columnIndex => this._columnIndex;
  int _columnIndex;

  /// The value of a cell. Typically it is from underlying datasource.
  Object get cellValue => this._cellValue;
  Object _cellValue;

  /// The display text of a cell. Typically it is the formatted text when number
  /// format or date format is applied for column.
  String get displayText => this._displayText;
  String _displayText;

  /// The display text of a cell. Typically it is the formatted text when number
  /// format or date format is applied for column.
  set displayText(String newValue) => this._displayText = newValue;

  /// Decides how the style is applied when selection and cell style are
  /// applied.
  StylePreference get stylePreference => this._stylePreference;
  StylePreference _stylePreference = StylePreference.selection;

  /// Decides how the style is applied when selection and cell style are
  /// applied.
  set stylePreference(StylePreference stylePreference) {
    this._stylePreference = stylePreference;
  }
}
