part of datagrid;

/// A base class which provides functionalities for [DataCell].
abstract class DataCellBase {
  GridCell _columnElement;

  Key _key;

  DataGridCellStyle _cellStyle;

  GridCellRendererBase _renderer;

  CellType _cellType;

  bool _isVisible = true;

  bool _isEnsured = false;

  DataRowBase _dataRow;

  bool _isDirty = false;

  String _displayText;

  /// The column index of the [DataCell].
  int get columnIndex => this._columnIndex;
  int _columnIndex = -1;

  /// The column index of the [DataCell].
  set columnIndex(int value) => this._columnIndex = value;

  /// The row index of the [DataCell].
  int get rowIndex => this._rowIndex;
  int _rowIndex = -1;

  /// The row index of the [DataCell].
  set rowIndex(int newValue) => this._rowIndex = newValue;

  /// [GridColumn] which is associated with [DataCell].
  GridColumn get gridColumn => this._gridColumn;
  GridColumn _gridColumn;

  /// [GridColumn] which is associated with [DataCell].
  set gridColumn(GridColumn value) => this._gridColumn = value;

  /// The cell value of the column element associated with the [DataCell].
  Object get cellValue => this._cellValue;
  Object _cellValue;

  /// The cell value of the column element associated with the [DataCell].
  set cellValue(Object newCellValue) {
    _cellValue = newCellValue;
  }

  /// Decides whether the [DataCell] is visible.
  bool get isVisible => this._isVisible;

  GridCell _onInitializeColumnElement(bool isInEdit) {
    return null;
  }

  void _updateColumn() {}

  void _onTouchUp() {
    if (this._dataRow != null) {
      final _DataGridSettings dataGridSettings =
          this._dataRow._dataGridStateDetails();
      if (this.rowIndex <=
              _GridIndexResolver.getHeaderIndex(dataGridSettings) ||
          dataGridSettings.selectionMode == SelectionMode.none) return;

      final RowColumnIndex currentRowColumnIndex =
          RowColumnIndex(this.rowIndex, this.columnIndex);
      dataGridSettings.rowSelectionManager.handleTap(currentRowColumnIndex);
    }
  }

  /// Decides whether the [DataCell] has the currentcell.
  bool get isCurrentCell => this._isCurrentCell;
  bool _isCurrentCell = false;

  /// Decides whether the [DataCell] has the currentcell.
  set isCurrentCell(bool isCurrentCell) {
    this._isCurrentCell = isCurrentCell;
  }
}
