part of datagrid;

/// A base class which provides functionalities for [DataRow].
abstract class DataRowBase {
  _DataGridStateDetails _dataGridStateDetails;

  Key _key;

  bool _isDirty = false;

  bool _isEnsured = false;

  bool _isVisible = true;

  List<DataCellBase> _visibleColumns = [];

  /// The row index of the [DataRow].
  int get rowIndex => this._rowIndex;
  int _rowIndex = -1;

  /// The row index of the [DataRow].
  set rowIndex(int newValue) => this._rowIndex = newValue;

  /// The type of the [DataRow].
  RowType get rowType => this._rowType;
  RowType _rowType = RowType.headerRow;

  /// The type of the [DataRow].
  set rowType(RowType newValue) => this._rowType = newValue;

  /// Decides whether the [DataRow] is visible.
  bool get isVisible => this._isVisible;

  /// The style of the row which is set through [SfDataGrid.onQueryRowStyle].
  DataGridCellStyle get rowStyle => this._rowStyle;
  DataGridCellStyle _rowStyle;

  /// The style of the row which is set through [SfDataGrid.onQueryRowStyle].
  set rowStyle(DataGridCellStyle newStyle) => this._rowStyle = newStyle;

  /// The region of the [DataRow] to identify whether the row is scrollable.
  RowRegion get rowRegion => this._rowRegion;
  RowRegion _rowRegion = RowRegion.header;

  /// The region of the [DataRow] to identify whether the row is scrollable.
  set rowRegion(RowRegion newValue) => this._rowRegion = newValue;

  void _rowIndexChanged() {
    if (this.rowIndex < 0) {
      return;
    }

    this._visibleColumns.forEach((col) {
      col.rowIndex = this.rowIndex;
      col._dataRow = this;
      col._updateColumn();
    });
  }

  void _onGenerateVisibleColumns(_VisibleLinesCollection visibleColumnLines) {}

  //Required and currently used properties
  void _initializeDataRow(_VisibleLinesCollection visibleColumnLines) {
    this._onGenerateVisibleColumns(visibleColumnLines);
  }

  void _ensureColumns(_VisibleLinesCollection visibleColumnLines) {}

  // Used in implementation
  _VisibleLineInfo _getColumnVisibleLineInfo(int index) {
    return this
        ._dataGridStateDetails()
        .container
        .scrollColumns
        .getVisibleLineAtLineIndex(index);
  }

  double _getColumnSize(int index, bool lineNull) {
    if (lineNull) {
      var currentPos = this
          ._dataGridStateDetails()
          .container
          .scrollColumns
          .rangeToRegionPoints(index, index, true);
      return currentPos[1].length;
    }

    var line = this._getColumnVisibleLineInfo(index);
    if (line == null) {
      return 0;
    }

    return line.size;
  }

  /// Decides whether the [DataRow] is selected.
  bool get isSelectedRow => this._isSelectedRow;
  bool _isSelectedRow = false;

  /// Decides whether the [DataRow] is selected.
  set isSelectedRow(bool newValue) {
    if (this._isSelectedRow != newValue) {
      this._isSelectedRow = newValue;
      this._visibleColumns?.forEach((dataCell) {
        dataCell._isDirty = true;
        dataCell._updateColumn();
      });
    }
  }

  StylePreference _stylePreference = StylePreference.selection;

  /// Decides whether the [DataRow] is currently active.
  bool get isCurrentRow => this._isCurrentRow;
  bool _isCurrentRow = false;

  /// Decides whether the [DataRow] is currently active.
  set isCurrentRow(bool isCurrentRow) {
    this._isCurrentRow = isCurrentRow;
  }
}
