part of datagrid;

class _VisualContainer extends StatefulWidget {
  const _VisualContainer(
      {Key key, this.rowGenerator, this.containerSize, this.isDirty})
      : super(key: key);

  final Size containerSize;
  final _RowGenerator rowGenerator;
  final bool isDirty;

  @override
  State<StatefulWidget> createState() => _VisualContainerState();
}

class _VisualContainerState extends State<_VisualContainer> {
  @override
  Widget build(BuildContext context) {
    final List<Widget> children = widget.rowGenerator.items
        .where((row) => row.rowRegion == RowRegion.body)
        .map<Widget>((dataRow) {
      return _VirtualizingCellsWidget(
        key: dataRow._key,
        dataRow: dataRow,
        isDirty: widget.isDirty || dataRow._isDirty,
      );
    }).toList(growable: false);

    return _VisualContainerRenderObjectWidget(
      key: widget.key,
      children: children,
      containerSize: widget.containerSize,
      isDirty: widget.isDirty,
    );
  }
}

class _VisualContainerRenderObjectWidget extends MultiChildRenderObjectWidget {
  _VisualContainerRenderObjectWidget(
      {Key key, this.containerSize, this.isDirty, this.children})
      : super(key: key, children: RepaintBoundary.wrapAll(List.from(children)));

  final List<Widget> children;
  final Size containerSize;
  final bool isDirty;

  @override
  _RenderVisualContainer createRenderObject(BuildContext context) {
    return _RenderVisualContainer(
        containerSize: this.containerSize, isDirty: this.isDirty);
  }

  @override
  void updateRenderObject(
      BuildContext context, _RenderVisualContainer renderObject) {
    super.updateRenderObject(context, renderObject);
    renderObject
      ..containerSize = this.containerSize
      ..isDirty = this.isDirty;
  }
}

class _VisualContainerParentData extends ContainerBoxParentData<RenderBox> {
  double width;
  double height;
  _VisualContainerParentData({this.width, this.height});

  void reset() {
    this.width = 0.0;
    this.height = 0.0;
    this.offset = Offset.zero;
  }
}

class _RenderVisualContainer extends RenderBox
    with
        ContainerRenderObjectMixin<RenderBox, _VisualContainerParentData>,
        RenderBoxContainerDefaultsMixin<RenderBox, _VisualContainerParentData> {
  bool get isDirty => this._isDirty;
  bool _isDirty = false;

  set isDirty(bool newValue) {
    this._isDirty = newValue;
    if (this._isDirty) {
      this.markNeedsLayout();
      this.markNeedsPaint();
    }
  }

  Size get containerSize => this._containerSize;
  Size _containerSize = Size.zero;

  set containerSize(Size newContainerSize) {
    if (this._containerSize == newContainerSize) return;
    this._containerSize = newContainerSize;
    this.markNeedsLayout();
    this.markNeedsPaint();
  }

  _RenderVisualContainer(
      {List<RenderBox> children, Size containerSize, bool isDirty})
      : this._containerSize = containerSize,
        this._isDirty = isDirty {
    addAll(children);
  }

  @override
  bool hitTestChildren(BoxHitTestResult result, {Offset position}) {
    return defaultHitTestChildren(result, position: position);
  }

  @override
  void setupParentData(RenderObject child) {
    super.setupParentData(child);
    if (child.parentData is! _VisualContainerParentData)
      child.parentData = _VisualContainerParentData();
  }

  @override
  bool get isRepaintBoundary => true;

  @override
  void performLayout() {
    size = constraints
        .constrain(Size(this.containerSize.width, this.containerSize.height));

    void _layout({RenderBox child, double width, double height}) {
      child.layout(BoxConstraints.tightFor(width: width, height: height),
          parentUsesSize: true);
    }

    RenderBox child = this.firstChild;
    while (child != null) {
      final _VisualContainerParentData _parentData = child.parentData;
      final wholeRowElement = child as _RenderVirtualizingCellsWidget;
      if (wholeRowElement.dataRow.isVisible) {
        final Rect rowRect = wholeRowElement.rowRect;
        _parentData.width = rowRect.width;
        _parentData.height = rowRect.height;
        _layout(child: child, width: size.width, height: _parentData.height);
        _parentData.offset = Offset(rowRect.left, rowRect.top);
      } else {
        child.layout(BoxConstraints.tightFor(width: 0.0, height: 0.0),
            parentUsesSize: false);
        _parentData.reset();
      }

      child = _parentData.nextSibling;
    }
  }

  @override
  void paint(PaintingContext context, Offset offset) {
    RenderBox child = firstChild;
    while (child != null) {
      final _VisualContainerParentData childParentData = child.parentData;
      if (childParentData.width != 0.0 && childParentData.height != 0.0) {
        context.paintChild(child, childParentData.offset + offset);
      }

      child = childParentData.nextSibling;
    }
  }
}
