part of datagrid;

class _CurrentCellManager {
  _DataGridStateDetails _dataGridStateDetails;

  int get rowIndex => this._rowIndex;
  int _rowIndex = -1;

  set rowIndex(int rowIndex) {
    this._rowIndex = rowIndex;
  }

  int get columnIndex => this._columnIndex;
  int _columnIndex = -1;

  set columnIndex(int columnIndex) {
    this._columnIndex = columnIndex;
  }

  _CurrentCellManager(_DataGridStateDetails dataGridStateDetails) {
    this._dataGridStateDetails = dataGridStateDetails;
  }

  bool _handlePointerOperation(
      _DataGridSettings dataGridSettings, RowColumnIndex rowColumnIndex) {
    var previousRowColumnIndex =
        RowColumnIndex(this.rowIndex, this.columnIndex);
    if (rowColumnIndex != previousRowColumnIndex &&
        dataGridSettings.navigationMode != GridNavigationMode.row) {
      if (!this._raiseCurrentCellActivating(rowColumnIndex)) return false;
      this._setCurrentCell(dataGridSettings, rowColumnIndex.rowIndex,
          rowColumnIndex.columnIndex);
      this._raiseCurrentCellActivated(previousRowColumnIndex);
    } else if (dataGridSettings.navigationMode == GridNavigationMode.row &&
        this.rowIndex != rowColumnIndex.rowIndex) {
      this._updateCurrentRowColumnIndex(
          rowColumnIndex.rowIndex, rowColumnIndex.columnIndex);
      this._updateBorderForMultipleSelection(dataGridSettings,
          previousRowColumnIndex: previousRowColumnIndex,
          nextRowColumnIndex: rowColumnIndex);
    }

    return true;
  }

  void _setCurrentCell(
      _DataGridSettings dataGridSettings, int rowIndex, int columnIndex,
      [bool needToUpdateColumn = true]) {
    if (this.rowIndex == rowIndex && this.columnIndex == columnIndex) return;

    this._removeCurrentCell(dataGridSettings, needToUpdateColumn);
    this._updateCurrentRowColumnIndex(rowIndex, columnIndex);
    var dataRowBase = _getDataRow(dataGridSettings, rowIndex);
    if (dataRowBase != null && needToUpdateColumn) {
      var dataCellBase = _getDataCell(dataRowBase, columnIndex);
      if (dataCellBase != null) {
        this._setCurrentCellDirty(dataRowBase, dataCellBase, true);
        dataCellBase._updateColumn();
      }
    }

    dataGridSettings.controller?._currentCell =
        _GridIndexResolver.resolveToRowColumnIndex(
            dataGridSettings, RowColumnIndex(rowIndex, columnIndex));
  }

  void _removeCurrentCell(_DataGridSettings dataGridSettings,
      [bool needToUpdateColumn = true]) {
    if (this.rowIndex == -1 && this.columnIndex == -1) return;

    var dataRowBase = _getDataRow(dataGridSettings, this.rowIndex);
    if (dataRowBase != null && needToUpdateColumn) {
      var dataCellBase = _getDataCell(dataRowBase, this.columnIndex);
      if (dataCellBase != null) {
        this._setCurrentCellDirty(dataRowBase, dataCellBase, false);
        dataCellBase._updateColumn();
      }
    }

    this._updateCurrentRowColumnIndex(-1, -1);
    dataGridSettings.controller?._currentCell = RowColumnIndex(-1, -1);
  }

  DataRowBase _getDataRow(_DataGridSettings dataGridSettings, int rowIndex) {
    var dataRows = dataGridSettings.rowGenerator.items;
    if (dataRows.isEmpty) return null;

    var dr = dataRows.where((row) => row.rowIndex == rowIndex);
    return dr.isNotEmpty ? dr.toList(growable: false).first : null;
  }

  DataCellBase _getDataCell(DataRowBase dataRow, int columnIndex) {
    if (dataRow._visibleColumns.isEmpty) return null;

    var dc = dataRow._visibleColumns
        .where((dataCell) => dataCell.columnIndex == columnIndex);
    return dc.isNotEmpty ? dc.toList(growable: false).first : null;
  }

  void _updateCurrentRowColumnIndex(int rowIndex, int columnIndex) {
    this._rowIndex = rowIndex;
    this._columnIndex = columnIndex;
  }

  void _setCurrentCellDirty(
      DataRowBase dataRow, DataCellBase dataCell, bool enableCurrentCell) {
    dataCell?.isCurrentCell = enableCurrentCell;
    dataCell?._isDirty = true;
    dataRow?.isCurrentRow = enableCurrentCell;
    dataRow?._isDirty = true;
  }

  bool _raiseCurrentCellActivating(RowColumnIndex rowColumnIndex) {
    final _DataGridSettings dataGridSettings = this._dataGridStateDetails();
    if (dataGridSettings.onCurrentCellActivating == null) return true;

    var newRowColumnIndex = _GridIndexResolver.resolveToRowColumnIndex(
        dataGridSettings, rowColumnIndex);
    var oldRowColumnIndex = _GridIndexResolver.resolveToRowColumnIndex(
        dataGridSettings, RowColumnIndex(this.rowIndex, this.columnIndex));
    return dataGridSettings.onCurrentCellActivating(
            newRowColumnIndex, oldRowColumnIndex) ??
        true;
  }

  void _raiseCurrentCellActivated(RowColumnIndex previousRowColumnIndex) {
    final _DataGridSettings dataGridSettings = this._dataGridStateDetails();
    if (dataGridSettings.onCurrentCellActivated == null) return;

    var newRowColumnIndex = _GridIndexResolver.resolveToRowColumnIndex(
        dataGridSettings, RowColumnIndex(this.rowIndex, this.columnIndex));
    var oldRowColumnIndex = _GridIndexResolver.resolveToRowColumnIndex(
        dataGridSettings, previousRowColumnIndex);
    dataGridSettings.onCurrentCellActivated(
        newRowColumnIndex, oldRowColumnIndex);
  }

  void _moveCurrentCellTo(
      _DataGridSettings dataGridSettings, RowColumnIndex nextRowColumnIndex,
      {bool isSelectionChanged = false, bool needToUpdateColumn = true}) {
    var previousRowColumnIndex = RowColumnIndex(
        dataGridSettings.currentCell.rowIndex,
        dataGridSettings.currentCell.columnIndex);

    this._scrollVertical(dataGridSettings, nextRowColumnIndex);
    this._scrollHorizontal(dataGridSettings, nextRowColumnIndex);

    if (dataGridSettings.navigationMode == GridNavigationMode.cell)
      this._setCurrentCell(dataGridSettings, nextRowColumnIndex.rowIndex,
          nextRowColumnIndex.columnIndex, needToUpdateColumn);
    else
      this._updateCurrentRowColumnIndex(
          nextRowColumnIndex.rowIndex, nextRowColumnIndex.columnIndex);

    if (dataGridSettings.selectionMode != SelectionMode.none &&
        dataGridSettings.selectionMode != SelectionMode.multiple &&
        !isSelectionChanged) {
      var rowSelectionController =
          dataGridSettings.rowSelectionManager as RowSelectionManager;
      if (rowSelectionController != null) {
        rowSelectionController._processSelection(
            dataGridSettings, nextRowColumnIndex, previousRowColumnIndex);
        rowSelectionController._pressedRowColumnIndex = nextRowColumnIndex;
      }
    }
  }

  void _processCurrentCell(
      _DataGridSettings dataGridSettings, RowColumnIndex rowColumnIndex,
      {bool isSelectionChanged = false}) {
    if (dataGridSettings.navigationMode == GridNavigationMode.row) {
      this._moveCurrentCellTo(dataGridSettings, rowColumnIndex,
          isSelectionChanged: isSelectionChanged);
      return;
    }

    if (this._raiseCurrentCellActivating(rowColumnIndex)) {
      this._moveCurrentCellTo(dataGridSettings, rowColumnIndex,
          isSelectionChanged: isSelectionChanged);
      this._raiseCurrentCellActivated(rowColumnIndex);
    }
  }

  void _scrollHorizontal(
      _DataGridSettings dataGridSettings, RowColumnIndex rowColumnIndex) {
    if (rowColumnIndex.columnIndex < this.columnIndex) {
      if (_SelectionHelper.needToScrollLeft(dataGridSettings, rowColumnIndex)) {
        _SelectionHelper.scrollInViewFromRight(dataGridSettings,
            previousCellIndex: rowColumnIndex.columnIndex);
      }
    }

    if (rowColumnIndex.columnIndex > this.columnIndex) {
      if (_SelectionHelper.needToScrollRight(
          dataGridSettings, rowColumnIndex)) {
        _SelectionHelper.scrollInViewFromLeft(dataGridSettings,
            nextCellIndex: rowColumnIndex.columnIndex);
      }
    }
  }

  void _scrollVertical(
      _DataGridSettings dataGridSettings, RowColumnIndex rowColumnIndex) {
    if (rowColumnIndex.rowIndex < this.rowIndex) {
      if (_SelectionHelper.needToScrollUp(
          dataGridSettings, rowColumnIndex.rowIndex)) {
        _SelectionHelper.scrollInViewFromDown(dataGridSettings,
            previousRowIndex: rowColumnIndex.rowIndex);
      }
    }

    if (rowColumnIndex.rowIndex > this.rowIndex) {
      if (_SelectionHelper.needToScrollDown(
          dataGridSettings, rowColumnIndex.rowIndex)) {
        _SelectionHelper.scrollInViewFromTop(dataGridSettings,
            nextRowIndex: rowColumnIndex.rowIndex);
      }
    }
  }

  void _updateBorderForMultipleSelection(_DataGridSettings dataGridSettings,
      {RowColumnIndex previousRowColumnIndex,
      RowColumnIndex nextRowColumnIndex}) {
    if (kIsWeb &&
        dataGridSettings.navigationMode == GridNavigationMode.row &&
        dataGridSettings.selectionMode == SelectionMode.multiple) {
      if (previousRowColumnIndex != null)
        dataGridSettings.currentCell
            ._getDataRow(dataGridSettings, previousRowColumnIndex.rowIndex)
            ?._isDirty = true;

      var firstVisibleColumnIndex =
          _GridIndexResolver.resolveToStartColumnIndex(dataGridSettings);
      this._updateCurrentRowColumnIndex(
          nextRowColumnIndex?.rowIndex ?? this.rowIndex,
          nextRowColumnIndex?.columnIndex ?? firstVisibleColumnIndex);
      dataGridSettings.currentCell
          ._getDataRow(
              dataGridSettings, nextRowColumnIndex?.rowIndex ?? this.rowIndex)
          ?._isDirty = true;
    }
  }

  @override
  bool operator ==(other) =>
      identical(this, other) ||
      other is _CurrentCellManager && runtimeType == other.runtimeType;

  @override
  int get hashCode => super.hashCode;
}
